#include <x86/exception.h>
         
.file "exception_wrappers.S"

.text

/* The address of the table of handlers (defined in exception.c) */
.extern exception_handler_array

/* The address of the table of wrappers (defined below, and shared
   with exception.c */
.globl exception_wrapper_array

	
/**
 * For exceptions with/without error code, refer to Intel x86 doc vol 3,
 * section 5.12
 */

/* These wrappers are for exceptions without error code */
.irp id,                              \
         EXCEPT_DIVIDE_ERROR,                \
         EXCEPT_DEBUG,                       \
         EXCEPT_NMI_INTERRUPT,               \
         EXCEPT_BREAKPOINT,                  \
         EXCEPT_OVERFLOW,                    \
         EXCEPT_BOUND_RANGE_EXCEDEED,        \
         EXCEPT_INVALID_OPCODE,              \
         EXCEPT_DEVICE_NOT_AVAILABLE,        \
         EXCEPT_COPROCESSOR_SEGMENT_OVERRUN, \
         EXCEPT_INTEL_RESERVED_1,            \
         EXCEPT_FLOATING_POINT_ERROR,        \
         EXCEPT_MACHINE_CHECK,               \
         EXCEPT_INTEL_RESERVED_2,            \
         EXCEPT_INTEL_RESERVED_3,            \
         EXCEPT_INTEL_RESERVED_4,            \
         EXCEPT_INTEL_RESERVED_5,            \
         EXCEPT_INTEL_RESERVED_6,            \
         EXCEPT_INTEL_RESERVED_7,            \
         EXCEPT_INTEL_RESERVED_8,            \
         EXCEPT_INTEL_RESERVED_9,            \
         EXCEPT_INTEL_RESERVED_10,           \
         EXCEPT_INTEL_RESERVED_11,           \
         EXCEPT_INTEL_RESERVED_12,           \
         EXCEPT_INTEL_RESERVED_13,           \
         EXCEPT_INTEL_RESERVED_14
 
        .p2align 2, 0x90
        exception_wrapper_\id:
        .type exception_wrapper_\id,@function
 
                /* Fake error code */
                pushl $0
                /* Backup the context */
                pushl %ebp
                movl %esp, %ebp
 
                pushl %edi
                pushl %esi
                pushl %edx
                pushl %ecx
                pushl %ebx
                pushl %eax
                subl  $2,%esp
                pushw %ss
                pushw %ds
                pushw %es
                pushw %fs
                pushw %gs
 
                /* Call the handler with exception number as
                 * argument */
                pushl $\id
                leal  exception_handler_array,%edi
                call  *\id*4(%edi)
                addl  $4, %esp

                /* Restore the context */
                popw  %gs
                popw  %fs
                popw  %es
                popw  %ds
                popw  %ss
                addl  $2,%esp
                popl  %eax
                popl  %ebx
                popl  %ecx
                popl  %edx
                popl  %esi
                popl  %edi
 
                popl  %ebp
                /* Remove fake error code */
                addl $4, %esp
                iret
.endr 
 
        /* These wrappers are for exceptions with error code */
.irp id,                              \
	EXCEPT_INVALID_TSS,                  \
	EXCEPT_SEGMENT_NOT_PRESENT,          \
	EXCEPT_STACK_SEGMENT_FAULT,          \
	EXCEPT_GENERAL_PROTECTION,           \
	EXCEPT_PAGE_FAULT,                   \
	EXCEPT_ALIGNEMENT_CHECK
 
        .p2align 2, 0x90
        exception_wrapper_\id:
        .type exception_wrapper_\id,@function
 
                /* ret eflags */
                /* ret cs */
                /* ret eip */
                /* Error code */
 
                /* Backup the context */
                pushl %ebp
                movl %esp, %ebp
 
                pushl %edi
                pushl %esi
                pushl %edx
                pushl %ecx
                pushl %ebx
                pushl %eax
                subl  $2,%esp
                pushw %ss
                pushw %ds
                pushw %es
                pushw %fs
                pushw %gs

                /* Call the handler with exception number as
                 * argument */
                pushl $\id
                leal  exception_handler_array,%edi
                call  *\id*4(%edi)
                addl  $4, %esp

                /* Restore the context */
                popw  %gs
                popw  %fs
                popw  %es
                popw  %ds
                popw  %ss
                addl  $2,%esp
                popl  %eax
                popl  %ebx
                popl  %ecx
                popl  %edx
                popl  %esi
                popl  %edi
                popl  %ebp
 
                /* Error code isn't compatible with iretd */
                addl $4, %esp
 
                iret
.endr


/* Double fault handler not supported. We must define it since we
   define an entry for it in the exception_wrapper_array. */
.irp id, EXCEPT_DOUBLE_FAULT
.p2align 2, 0x90
exception_wrapper_\id:
.type exception_wrapper_\id,@function
1:	hlt
	jmp 1b /* Machine halting */
.endr

/* Build the exception_wrapper_array, shared with interrupt.c */
.section ".rodata"
.p2align 5, 0x0
exception_wrapper_array:
	.irp id, 0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15, \
	         16,17,18,19,20,21,22,23,24,25,26,27,29,30,31
	  .long (sos_exception_wrapper_\id)
	.endr
