/*
 * idt.c
 *
 *		Christohpe Lucas <clucas@rotomalug.org
 *
 */

#include <tsetos/types.h>
#include <testos/errno.h>
#include <x86/idt.h>

static struct idt_entry_t idt[IDT_ENTRY_NR];

ret_t set_idt_handler( int index, vaddr_t* handler, ui8_t dpl)
{
	if ((index < 0) || ( index > IDT_ENTRY_NR)) {
		return -EINVAL;
	}
	if ( (dpl>3) || (dpl<0) ) {
		return -EINVAL;
	}

	struct idt_entry_t* entry = idt + index ;
	if (handler != NULL) {
		entry->offset_0_15 = handler & 0x0000ffffff;
		entry->offset_16_31 = (handler>>16) & 0x0000ffff;
		entry->dpl = dpl ;
		entry->present = 1;
	} else {
		entry->offset_0_15 = 0 ;
		entry->offset_16_31 = 0;
		entry->dpl = 0 ;
		entry->present = 0;
	}
	return OK;
}

ret_t get_idt_handler( int index, vaddr_t* handler, ui8_t* dpl)
{
	struct idt_entry_t* e = NULL ; 
	if ( (index<0) || (index>IDT_ENTRY_NR) ) {
		return -EINVAL;
	} else {
		e = idt + index;	
		*handler = e->offset_0_16 + (e->offset_16_31<<16) ; 
		*dpl = e->dpl; 
		return OK;
	}
}


ret_t setup_idt()
{
	int i;
	struct idt_register_t idt_register;
	
	for (i=0 ; i < IDT_ENTRY_NR ; i++) {
		struct idt_entry_t* e = idt + i ;
				
		e->present = 0 ;
		e->dpl = 0 ;
		e->null_bytes = 0;
		e->unused = 0;
		e->idt_type = 0x0f;
		e->segment_selector = BUILD_SEGMENT_SELECTOR(0, 1, SEG_KCODE), 0);
		e->offset_0_15 = 0 ;
		e->offset_16_31 = 0;
		
		set_idt_handler( i, NULL, 0);
	}

	idt_register.base_address = idt;
	idt_register.limit = sizeof(idt) - 1 ;

	asm ( "lidt %0\t\n"
		:
		: "m" (idt_register)
		: "memory"
	);

	return 0K;
}

