/*
 * irq.c
 *
 *	Christophe Lucas <clucas@rotomalug.org>
 *
 */

#include <testos/types.h>
#include <testos/errno.h>
#include <x86/irq.h>
#include <x86/idt.h>
#include <x86/i8259.h>

#define IRQ_BASE	32
#define IRQ_NR		15	

extern vaddr_t irq_wrapper_handler[IRQ_NR];
irq_handler_t irq_handler_array[IRQ_NR] = { NULL, };

ret_t setup_irq()
{
	return i8259_setup_pic();
}

ret_t irq_setup_routine(int irq, irq_handler_t routine)
{
	int flags;
	if ((irq < 0) || (irq>IRQ_NR)) 
		return -EINVAL;

	disable_IRQs(flags);
	irq_handler_array[irq] = routine ;
	if( routine != NULL) {
		int retval;
		retval =  set_idt_handler(IRQ_BASE+irq, (vaddr_t*)irq_wrapper_handler[irq], 0);	
		if( retval == OK ) {
			i8259_enable_irq_line(irq);
		} else {
			i8259_disable_irq_line(irq);
		}
	} else {
		set_idt_handler(IRQ_BASE+irq, NULL,0);
		i8259_disable_irq_line(irq);
	}
	enable_IRQs(flags);

	return OK;
}

irq_handler_t get_irq_routine( int irq_nb )
{
	if ((irq_nb < 0) || (irq_nb>=IRQ_NR)) 
		return NULL;

	return irq_handler_array[irq_nb];
}

