/*
 *  idt.h
 * 
 *  Christophe Lucas <cluacs@rotomalug.org>
 *
 */

#ifndef __IDT_H__
#define __IDT_H__

#include <testos/types.h>
#include <testos/errno.h>

#define IDT_ENTRY_NR		256

struct idt_register_t {
	ui32_t base_address;
	ui16_t limit;
} __attribute__((packed,aligned(8))) ;

struct idt_entry_t {
	ui16_t offset_0_15;
	ui16_t segment_selector;
	ui8_t unused:5;
	ui8_t null_bytes:3;
	ui8_t idt_type:5;	/* interrupt or trap gate */
	ui8_t dpl:2;
	ui8_t present:1;
	ui16_t offset_16_31;
} __attribute__((packed));

#define BUILD_INTERRUPT_IDT_ENTRY(offset,segment_selector,dpl)	\
	((struct idt_entry_t) {					\
		.offset_0_15 = offset & 0x0000ffff,		\
		.segment_selector = segment_selector,		\
		.unused = 0,					\
		.null_bytes = 0,				\
		.idt_type = 0x0f, /* 32 bits interrupt */	\
		.dpl = dpl,					\
		.present = 0,					\
		.offset_16_31 = offset & 0xffff0000		\
	})

#define BUILD_TRAP_IDT_ENTRY(offset,segment_selector,dpl)	\
	((struct idt_entry_t) {					\
		.offset_0_15 = offset & 0x0000ffff,		\
		.segment_selector = segment_selector,		\
		.unused = 0,					\
		.null_bytes = 0,				\
		.idt_type = 0x0e, /* 32 bits interrupt */	\
		.dpl = dpl,					\
		.present = 0,					\
		.offset_16_31 = (offset>>16) & 0xffff		\
	})

extern ret_t set_idt_handler( int index, vaddr_t* handler, ui8_t dpl);
extern ret_t get_idt_handler( int index, vaddr_t* handler, ui8_t* dpl);
extern ret_t setup_idt();
	

#endif
