#ifndef __IRQ_H__
#define __IRQ_H__
#include <testos/errno.h>

#define irq_save_flags(flags) \
  asm volatile("pushfl ; popl %0":"=g"(flags)::"memory")
#define irq_restore_flags(flags) \
  asm volatile("push %0; popfl"::"g"(flags):"memory")

#define disable_IRQs(flags)    \
  ({ irq_save_flags(flags); asm("cli\n"); })
#define restore_IRQs(flags)    \
  irq_restore_flags(flags)

/* Usual IRQ levels */
#define IRQ_TIMER         0
#define IRQ_KEYBOARD      1
#define IRQ_SLAVE_PIC     2
#define IRQ_COM2          3
#define IRQ_COM1          4
#define IRQ_LPT2          5
#define IRQ_FLOPPY        6
#define IRQ_LPT1          7
#define IRQ_8_NOT_DEFINED 8
#define IRQ_RESERVED_1    9
#define IRQ_RESERVED_2    10
#define IRQ_RESERVED_3    11
#define IRQ_RESERVED_4    12
#define IRQ_COPROCESSOR   13
#define IRQ_HARDDISK      14
#define IRQ_RESERVED_5    15

typedef void (*irq_handler_t)(int irq_level);

/* Setup the PIC */
ret_t setup_irq(void);

/**
 * If the routine is not NULL, the IDT is setup to call an IRQ
 * wrapper upon interrupt, which in turn will call the routine, and
 * the PIC is programmed to raise an irq.\ If the routine is
 * NULL, we disable the irq line.
 */
ret_t irq_set_routine(int irq_level, irq_handler_t routine);

irq_handler_t irq_get_routine(int irq_level);


#endif
